/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of rrd_agent.
	
    rrd_agnet is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    rrd_agent is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#include "config.h"
#include <string>
#include <boost/shared_ptr.hpp>
#ifndef _WIN32
#include <signal.h>
#include <string.h>
#include <unistd.h>
#endif


#include <log4cplus/logger.h>
#include <log4cplus/loglevel.h>
#include <log4cplus/configurator.h>
#include <log4cplus/layout.h>
#ifdef _WIN32
#include "XGetopt.h"
#include <log4cplus/nteventlogappender.h>
#else
#include <log4cplus/syslogappender.h>
#endif

bool load_config(const std::string & filename, std::string & rrd_path, std::string & rra_def_file,  std::string & endpoint);
void process_messages(const std::string & endpoint, const std::string & rrd_path, const std::string & rra_def_file);

#ifdef _WIN32
void service_main(int argc, char **argv);
bool install_service();
bool remove_service();
extern std::string service_name;
#endif

#ifndef _WIN32
sig_atomic_t run = 1;
#else
int run = 1;
#endif

std::string app_name(PACKAGE_NAME);
const std::string version(PACKAGE_VERSION);
std::string endpoint;
std::string rrd_path;
std::string rra_def_file;

#ifndef _WIN32
void sig_handler(int signum)
{
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	bool terminate = false;
	switch (signum) {
		case SIGHUP : 
			LOG4CPLUS_INFO(logger, "received SIGHUP");
			break;
		case SIGTERM:
			LOG4CPLUS_INFO(logger, "received SIGTERM");
			terminate=true;
			break;
		case SIGINT:
			LOG4CPLUS_INFO(logger, "received SIGINT");
			terminate=true;
			break;
		default:
			std::ostringstream log_message;
			log_message << "unhandled signal (" << strsignal(signum) << ")";
			LOG4CPLUS_INFO(logger, log_message); 
			break;
	}
	if(terminate) {
		LOG4CPLUS_INFO(logger, "closing"); 
		run=0;
	}
}

#endif




int main(int argc, char **argv)
{
	int opt;
	bool daemonise = false;
	std::string config_file;
	std::string logger_config;
#ifndef _WIN32
	config_file="/etc/rrd_agent/config/rrd_agent.conf";
	logger_config="/etc/rrd_agent/config/logger.conf";
#else
	config_file="C:\\Documents and Settings\\All Users\\Application Data\\rrd_agent\\rrd_agent.conf";
	logger_config="C:\\Documents and Settings\\All Users\\Application Data\\rrd_agent\\logger.conf";
#endif

#ifdef _WIN32
	while((opt=getopt(argc, argv, "dirf:"))!=-1){
#else
	while((opt=getopt(argc, argv, "df:"))!=-1){
#endif
		switch(opt) {
			case 'd':
				daemonise = true;
				break; 
			case 'f':
				config_file = optarg;
				break;
			case 'l':
				logger_config = optarg;
				break;

	#ifdef _WIN32
			case 'i':
				if(!install_service()) {
					std::cout << "service install failed" << std::endl;
					exit(EXIT_FAILURE);
				}
				std::cout << "service installed successfully" << std::endl;
				exit(EXIT_SUCCESS);
				break;
			case 'r':
				if(!remove_service()) {
					std::cout << "service remove failed" << std::endl;
					exit(EXIT_FAILURE);
				}
				std::cout << "service removed successfully" << std::endl;
				exit(EXIT_SUCCESS);
				break;
    #endif
		}
	}
	log4cplus::PropertyConfigurator::doConfigure(logger_config);
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	std::ostringstream log_message;
	log_message << "Version " << version;
	LOG4CPLUS_INFO(logger, log_message.str());
	LOG4CPLUS_INFO(logger, "starting");



#ifndef _WIN32
	rrd_path="/var/envi/rrds";
	rra_def_file="/etc/rrd_agent/config/rra.conf";
#else
	rrd_path="c:\\envi_data";
	rra_def_file="C:\\Documents and Settings\\All Users\\Application Data\\envi\\rrd_agent\\rra.conf";
#endif
	if(!load_config(config_file, rrd_path, rra_def_file, endpoint)) {
		exit(EXIT_FAILURE);
	}
	log_message.str("");
	log_message << "endpoint: " << endpoint;
	LOG4CPLUS_INFO(logger, log_message.str());
	
	
#ifdef _WIN32
	if(daemonise) {
		SERVICE_TABLE_ENTRY service_table[2];
		service_table[0].lpServiceName = "rrdAgent";
		service_table[0].lpServiceProc = (LPSERVICE_MAIN_FUNCTION)service_main;
		service_table[1].lpServiceName = NULL;
		service_table[1].lpServiceProc = NULL;
		LOG4CPLUS_INFO(logger, "starting service");
		StartServiceCtrlDispatcher(service_table);
		exit(EXIT_SUCCESS);
	}
	else {
		process_messages(endpoint, rrd_path, rra_def_file);
	}
#else
	signal(SIGHUP, sig_handler);
	signal(SIGTERM, sig_handler);
	signal(SIGINT, sig_handler);
	signal(SIGQUIT, sig_handler);
	if(daemonise) {
		LOG4CPLUS_INFO(logger, "run as daemon");
		daemon(0, 0);
	}
	process_messages(endpoint, rrd_path, rra_def_file);
#endif
	
	LOG4CPLUS_INFO(logger, "closed");
	logger.shutdown();
	exit(EXIT_SUCCESS);
}

